/* Copyright (c) 2006 Yahoo! Inc. All rights reserved. */
/* Container Styles */

.calendar {
    font-size: 0.9em;
    margin-top: 0;
}

.calcontainer {*height:1%;} /* IE */
.calcontainer:after {content:'.';clear:both;display:block;visibility:hidden;height:0;} /* others */

.calbordered {
	float:left;
	padding:5px;
	background-color:#F7F9FB;
	border:1px solid #7B9EBD;
}

.calbordered .title {
	font-family: Arial,Helvetica,sans-serif;
	color:#000;
	font-weight:bold;
	margin-bottom:5px;
	height:auto;
	width:304px;
	position:relative;
}

.title .close-icon {
	position:absolute;
	right:0;
	top:0;
	border:none;
}

.cal2up {
	float:left;
}

.calnavleft {
	position:absolute;
	top:0;
	bottom:0;
	height:12px;   
	left:2px;
}

.calnavright {
	position:absolute;
	top:0;
	bottom:0;
	height:12px;  
	right:2px;
}

/* Calendar element styles */

.calendar {
	font-family: Arial,Helvetica,sans-serif;
	text-align:center;
	border-spacing:0;
}

td.calcell {
	width:1.5em;
	height:1em;
	border:1px solid #E0E0E0;
	background-color:#FFF;
}

.calendar.wait td.calcell {
	color:#999;
	background-color:#CCC;
}

.calendar.wait td.calcell a {
	color:#999;
	font-style:italic;
}

td.calcell a {
	color:#003DB8;
	text-decoration:none;
}

td.calcell.today {
	border:1px solid #000;
}

td.calcell.oom {
	cursor:default;
	color:#999;
	background-color:#EEE;
	border:1px solid #E0E0E0;
}

td.calcell.selected {
	color:#003DB8;
	background-color:#FFF19F;
	border:1px solid #FF9900;
}

td.calcell.calcellhover {
	cursor:pointer;
	color:#FFF;
	background-color:#FF9900;
	border:1px solid #FF9900;
}

/* Added to perform some correction for Opera 8.5
   hover redraw bug */
.calendar table:hover {
	background-color:#FFF;
}

td.calcell.calcellhover a {
	color:#FFF;
}

td.calcell.restricted {
	text-decoration:line-through;
}

td.calcell.previous {
	color:#CCC;
}

td.calcell.highlight1 { background-color:#CCFF99; }
td.calcell.highlight2 { background-color:#99CCFF; }
td.calcell.highlight3 { background-color:#FFCCCC; }
td.calcell.highlight4 { background-color:#CCFF99; }


.calhead {
	border:1px solid #E0E0E0;
	vertical-align:middle;
	background-color:#FFF;
}

.calheader {
	position:relative;
	width:100%;
}

.calheader img {
	border:none;
}

.calweekdaycell {
	color:#666;
	font-weight:normal;
}

.calfoot {
	background-color:#EEE;
}

.calrowhead, .calrowfoot {
	color:#666;
	font-size:9px;
	font-style:italic;
	font-weight:normal;
	width:15px;
}

.calrowhead {
	border-right-width:2px;
}
