.calendar {
    text-align: center;
    margin: 2em 0 0 0;
}

.calendar td, .calendar th { padding: 0.1em 0.25em 0.1em 0.25em; }

.calendar caption .month {
    padding: 0 1em 0 1em;
    font-size: 1.5em;
}
