use strict;
use warnings;

package Jifty::Plugin::Nothing;
use base qw/Jifty::Plugin/;

# Your plugin goes here.  If takes any configuration or arguments, you
# probably want to override L<Jifty::Plugin/init>.

1;
